package Net::BGP::RIB;

use strict;
use vars qw( $VERSION );

## Inheritance and Versioning ##

$VERSION = '0.02';

## POD ##

=pod

=head1 NAME

Net::BGP::RIB - Class encapsulating a BGP Routing Information Base (RIB)

=head1 SYNOPSIS

    use Net::BGP::RIB;

=head1 DESCRIPTION

This module does not exist. It is only included as a place-holder in the
CPAN namespace for potential future enhancements to the B<Net::BGP> module.

=head1 METHODS

No methods are currently defined.

=head1 SEE ALSO

B<Net::BGP>, B<Net::BGP::Process>, B<Net::BGP::Peer>, B<Net::BGP::Update>,
B<Net::BGP::Notification>

=head1 AUTHOR

Stephen J. Scheck <code@neurosphere.com>

=cut
